#!/bin/sh
# Next line a tcl comment \
exec wish -f "$0" ${1+"$@"}

# HypScwrl2.tcl - updated version in THe BuScr 1.06a
# Distributed with THe BuScr 1.06a from structbio.biochem.dal.ca/jrainey
# Written by Jan K. Rainey - jan.rainey@dal.ca
# This script takes a PDB file output by THe BuScr containing all-atom side-
# chain predictions of HYP and merges it with a PDB file produced by SCWRL
# containing all-atom predictions of other amino acids
# Note: if output files from THe BuScr are "fed" to SCWRL, predicted PRO
# side-chains in X position and ALA side-chains in X & Y positions will be
# those produced by THe BuScr - they won't be modified by SCWRL

# To use: The only line that needs to be modified is the very last line
#         It is marked.
# Run the script in the directory that contains the PDB files or else
# specify the full path to each file.

# See manual for full process to use THe BuScr together with SCWRL

# Updated from HypScwrl.tcl to provide more versatile handling of PDB files
# with multiple non-ATOM line types

proc findnextATOM { fid } {

	set readflag 0
	
	while {$readflag == 0} {
	  if {[gets $fid currline] >=0} {
  		if {[string compare -length 4 {ATOM} $currline] == 0} {
			  return $currline
			}
		} else {
		  set readflag 1
			return {EOF}
		}
	}
}

proc outtopdb { fid AtPre At AtNum Res Chain ResNum xco yco zco } {
	 
	 set atid [string index $At 0]
   puts $fid [format {%-6s%5i %1s%-3s%4s%2s%4i    %8.3f%8.3f%8.3f%6.2f%6.2f%12s  } ATOM $AtNum $AtPre $At $Res $Chain $ResNum $xco $yco $zco 1.0 0.0 $atid]
		 
}

proc amalgamatePDB { buscrfile scwrlfile outfile } {

  set realAtNum 0
	set oldResNum -1
	set doneflag 0
	set buscrid [open $buscrfile r]
	set scwrlid [open $scwrlfile r]
	set outid [open $outfile w]

	# get first atom of each file
  set buscrline [findnextATOM $buscrid]
	if {$buscrline == {EOF}} {
	  incr doneflag
	} else {
		scan $buscrline "%*6s%5i%*c%c%3s%c%3s%*c%c%4i%c%*c%*c%*c%8f%8f%8f%6f%6f" BAtNum BAtPre BAt BAltLoc BRes BChain BResNum BAChar Bxco Byco Bzco BOcc BBFac
	}
  set scwrlline [findnextATOM $scwrlid]
	if {$scwrlline == {EOF}} {
	  incr doneflag
	} else {
 		scan $scwrlline "%*6s%5i%*c%c%3s%c%3s%*c%c%4i%c%*c%*c%*c%8f%8f%8f%6f%6f" SAtNum SAtPre SAt SAltLoc SRes SChain SResNum SAChar Sxco Syco Szco SOcc SBFac
	}
	
	while {$doneflag < 2} {
		
		if {$BResNum != $oldResNum} {
		  set oldResNum $BResNum
		}
		# Use THeBuScr output if Hyp, otherwise SCWRL
		# Note: backbone (incl. Glys) + any Ala/Pro that were predicted in SCWRL file actually arise from THeBuScr
		
		if {$BRes == {HYP}} {
		  puts stdout "HYP (using THeBuScr): residue $BResNum"
			set outflag {B}
		} else {
		  puts stdout "$SRes (using SCWRL unless Gly or Ala): residue $SResNum"
		  set outflag {S}
		}

		set olddoneflag $doneflag
    while {$BResNum == $oldResNum && $olddoneflag == $doneflag} {
			if {$outflag == {B}} {
        incr realAtNum
	      set BChain [format %c $BChain]
        set BAtPre [format %c $BAtPre]
        outtopdb $outid $BAtPre $BAt $realAtNum $BRes $BChain $BResNum $Bxco $Byco $Bzco
			}
  		set buscrline [findnextATOM $buscrid]
			if {$buscrline == {EOF}} {
	  		incr doneflag
			} else {
				scan $buscrline "%*6s%5i%*c%c%3s%c%3s%*c%c%4i%c%*c%*c%*c%8f%8f%8f%6f%6f" BAtNum BAtPre BAt BAltLoc BRes BChain BResNum BAChar Bxco Byco Bzco BOcc BBFac
			}
		}

		set olddoneflag $doneflag
    while {$SResNum == $oldResNum && $olddoneflag == $doneflag} {
			if {$outflag == {S}} {
        incr realAtNum
	      set SChain [format %c $SChain]
        set SAtPre [format %c $SAtPre]
			  outtopdb $outid $SAtPre $SAt $realAtNum $SRes $SChain $SResNum $Sxco $Syco $Szco
			}
  		set scwrlline [findnextATOM $scwrlid]
			if {$scwrlline == {EOF}} {
	  		incr doneflag
			} else {
 				scan $scwrlline "%*6s%5i%*c%c%3s%c%3s%*c%c%4i%c%*c%*c%*c%8f%8f%8f%6f%6f" SAtNum SAtPre SAt SAltLoc SRes SChain SResNum SAChar Sxco Syco Szco SOcc SBFac
			}
		}
	} 

	close $outid
	close $scwrlid
	close $buscrid
	
}


# -------------------------------- Main program --------------------------------

  # Modify the line below.
	# First argument: THe BuScr PDB output with HYP side-chains
	# Second argument: SCWRL PDB output - HYP will be replaced by GLY
	# Third argument: desired output file
	# Note: specify path to files if you are not running script in directory
	#       containing the files

  amalgamatePDB {/path/if/necessary/busrcout.pdb} {scwrlout.pdb} {amalgamated.pdb}
